package org.bitbucket.bradleysmithllc.etlunit.feature.oracle_database;

import org.bitbucket.bradleysmithllc.etlunit.Log;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.TestExecutionError;
import org.bitbucket.bradleysmithllc.etlunit.feature.database.*;
import org.bitbucket.bradleysmithllc.etlunit.feature.database.db.Table;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestValueObject;
import org.bitbucket.bradleysmithllc.etlunit.util.IOUtils;
import org.bitbucket.bradleysmithllc.etlunit.util.VelocityUtil;

import javax.inject.Inject;
import javax.inject.Named;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.sql.Connection;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;

public class OracleDatabaseImplementation extends BaseDatabaseImplemenation
{
	private Log applicationLog;
	private RuntimeSupport runtimeSupport;
	private DatabaseFeatureModule databaseFeatureModule;

	private static final int MASTER = 1;
	private static final int SCHEMA = 0;

	@Inject
	public void setLogger(@Named("applicationLog") Log log)
	{
		this.applicationLog = log;
	}

	@Inject
	public void setRuntimeSupport(RuntimeSupport runtimeSupport)
	{
		this.runtimeSupport = runtimeSupport;
	}

	@Inject
	public void setDatabaseFeatureModule(DatabaseFeatureModule databaseFeatureModule)
	{
		this.databaseFeatureModule = databaseFeatureModule;
	}

	public database_state getDatabaseState(DatabaseConnection databaseConnection, String s)
	{
		return database_state.pass;
	}

	public String getImplementationId()
	{
		return "oracle";
	}

	public Object processOperationSub(operation op, OperationRequest request) throws UnsupportedOperationException
	{
		switch (op)
		{
			case createDatabase:
				doCreateDatabase(request.getInitializeRequest());
				break;
		}

		return null;
	}

	private void executeScript(String script, DatabaseConnection databaseConnection, String mode, boolean master) throws IOException, TestExecutionError
	{
		final SQLAggregator sqlagg = databaseFeatureModule.resolveSQLRef(script);

		jdbcClient.useStatement(databaseConnection, mode, new JDBCClient.StatementClient()
		{
			public void connection(Connection conn, Statement st, DatabaseConnection connection, String mode, int id) throws Exception
			{
				SQLAggregator.Aggregator statementAggregator = sqlagg.getStatementAggregator();
				while (statementAggregator.hasNext())
				{
					String sql = statementAggregator.next().getLine();
					applicationLog.info(sql);
					st.execute(sql);
				}
			}
		}, master ? MASTER : SCHEMA);
	}

	private void doCreateDatabase(InitializeRequest initializeRequest)
	{
		Exception[] res = executeScripts(initializeRequest, new String[]{"kill_login", "kill_table_space", "kill_temp_table_space", "create_schema"});

		// make sure the create script did not fail.
		if (res[3] != null)
		{
			throw new IllegalStateException("Error creating schema", res[3]);
		}
	}

	private Exception [] executeScripts(InitializeRequest initializeRequest, String [] scripts)
	{
		DatabaseConnection databaseConnection = initializeRequest.getConnection();
		String mode = initializeRequest.getMode();

		String loginName = getLoginName(databaseConnection, mode, SCHEMA);
		String password = getPassword(databaseConnection, mode, SCHEMA);

		Map<String, String> databaseProperties = databaseConnection.getDatabaseProperties();

		String tablespace = loginName + "_FILE";
		String tempTablespace = "T_" + loginName + "_FILE";

		Map<String, String> map = new HashMap<String, String>();
		map.put("databaseName", loginName);
		map.put("databasePassword", password);

		if (databaseProperties != null)
		{
			String otableName = databaseProperties.get("tablespace");

			if (otableName != null)
			{
				tablespace = otableName;
				map.put("tablespaceSpecified", "true");
			}

			otableName = databaseProperties.get("temp-tablespace");

			if (otableName != null)
			{
				tempTablespace = otableName;
				map.put("tempTablespaceSpecified", "true");
			}
		}

		map.put("tablespace", tablespace);
		map.put("tempTablespace", tempTablespace);

		Exception [] exc_arr = new Exception[scripts.length];

		for (int index = 0; index < scripts.length; index++)
		{
			String script = scripts[index];

			URL url_script = getClass().getResource("/" + script + ".vm");

			try
			{
				String killText = IOUtils.readURLToString(url_script);

				String fscript = VelocityUtil.writeTemplate(killText, map);

				String scriptName = script + "_" + databaseConnection.getDatabaseName(mode) + ".sql";

				File scriptFile = runtimeSupport.createGeneratedSourceFile(getImplementationId(), scriptName);

				IOUtils.writeBufferToFile(scriptFile, new StringBuffer(fscript));

				executeScript(fscript, databaseConnection, mode, true);
			}
			catch (Exception e)
			{
				applicationLog.severe("Error executing script: " + e.toString(), e);
				exc_arr[index] = e;
			}
		}

		return exc_arr;
	}

	@Override
	protected String getJdbcUrl(DatabaseConnection dc, String mode, int id)
	{
		String serviceName = "";

		Map<String, String> databaseProperties = dc.getDatabaseProperties();

		if (databaseProperties != null)
		{
			String oserviceName = databaseProperties.get("service-name");
			if (oserviceName != null)
			{
				serviceName = "/" + oserviceName;
			}
		}

		return "jdbc:oracle:thin:@//" + dc.getServerName() + ":" + String.valueOf(dc.getServerPort() == -1 ? 1521 : dc.getServerPort()) + serviceName;
	}

	@Override
	protected Class getJdbcDriverClass()
	{
		return oracle.jdbc.OracleDriver.class;
	}

	@Override
	protected String getPassword(DatabaseConnection dc, String mode, int id)
	{
		return id == MASTER ? dc.getAdminPassword() : (IOUtils.hashToLength(super.getPassword(dc, mode, id), 28)).toUpperCase();
	}

	@Override
	protected String getLoginName(DatabaseConnection dc, String mode, int id)
	{
		return id == MASTER ? dc.getAdminUserName() : (IOUtils.hashToLength(super.getLoginName(dc, mode, id), 28)).toUpperCase();
	}

	public String getDefaultSchema(DatabaseConnection databaseConnection, String mode)
	{
		return getLoginName(databaseConnection, mode, SCHEMA);
	}

	@Override
	public boolean isTableTestVisible(DatabaseConnection dc, String mode, Table table) {
		String schema = table.getSchema().getName();

		return super.isTableTestVisible(dc, mode, table) && (schema == null || schema.equals(getLoginName(dc, mode, SCHEMA)));
	}
}
